@@ -1,4 +1,4 @@
-/* $Id: cpu.c,v 1.33 2004/03/10 00:15:39 bdejean Exp $ */
+/* $Id: cpu.c,v 1.36 2004/07/03 13:28:59 bdejean Exp $ */
 
 /* Copyright (C) 1998-99 Martin Baulig
    This file is part of LibGTop 1.0.
@@ -54,24 +54,17 @@ void
 glibtop_get_cpu_s (glibtop *server, glibtop_cpu *buf)
 {
 	char buffer [BUFSIZ], *p;
-	int fd, len, i;
-	guint64 total;
+	int i;
 
 	glibtop_init_s (&server, GLIBTOP_SYSDEPS_CPU, 0);
 
 	memset (buf, 0, sizeof (glibtop_cpu));
 
-	fd = open (FILENAME, O_RDONLY);
-	if (fd < 0)
-		glibtop_error_io_r (server, "open (%s)", FILENAME);
+	file_to_buffer(server, buffer, FILENAME);
 
-	len = read (fd, buffer, BUFSIZ-1);
-	if (len < 0)
-		glibtop_error_io_r (server, "read (%s)", FILENAME);
-
-	close (fd);
-
-	buffer [len] = '\0';
+	/*
+	 * GLOBAL
+	 */
 
 	p = skip_token (buffer);	/* "cpu" */
 
@@ -83,22 +76,26 @@ glibtop_get_cpu_s (glibtop *server, glibtop_cpu *buf)
 	/* 2.6 kernel */
 	buf->idle += strtoull(p, &p, 0); /* "iowait" */
 	buf->sys  += strtoull(p, &p, 0); /* "irq" */
-	buf->sys  += strtoull(p, &p, 0); /* "softirq" */	
+	buf->sys  += strtoull(p, &p, 0); /* "softirq" */
 
-	total = buf->user;
-	total += buf->nice;
-	total += buf->sys;
-	total += buf->idle;
-	buf->total = total;
+	buf->total = buf->user + buf->nice + buf->sys + buf->idle;
 
 	buf->frequency = 100;
 	buf->flags = _glibtop_sysdeps_cpu;
 
-	for (i = 0; i < server->ncpu; i++) {
-		if (strncmp (p+1, "cpu", 3) || !isdigit (p [4]))
+	/*
+	 * PER CPU
+	 */
+
+	for (i = 0; i < GLIBTOP_NCPU && i < server->ncpu; i++) {
+
+		p = skip_line(p); /* move to ^ */
+
+		if (strncmp (p, "cpu", 3) || !isdigit (p [3]))
 			break;
 
-		p += 6;
+		p = skip_token(p); /* "cpuN" */
+
 		buf->xcpu_user [i] = strtoull (p, &p, 0);
 		buf->xcpu_nice [i] = strtoull (p, &p, 0);
 		buf->xcpu_sys  [i] = strtoull (p, &p, 0);
@@ -109,13 +106,12 @@ glibtop_get_cpu_s (glibtop *server, glibtop_cpu *buf)
 		buf->xcpu_sys  [i]  += strtoull(p, &p, 0); /* "irq" */
 		buf->xcpu_sys  [i]  += strtoull(p, &p, 0); /* "softirq" */
 
-		total = buf->xcpu_user [i];
-		total += buf->xcpu_nice [i];
-		total += buf->xcpu_sys [i];
-		total += buf->xcpu_idle [i];
-
-		buf->xcpu_total [i] = total;
+		buf->xcpu_total[i] = buf->xcpu_user [i] \
+			+ buf->xcpu_nice [i] \
+			+ buf->xcpu_sys  [i] \
+			+ buf->xcpu_idle [i];
 	}
 
-	buf->flags |= _glibtop_sysdeps_cpu_smp;
+	if(i >= 2) /* ok, that's a real SMP */
+		buf->flags |= _glibtop_sysdeps_cpu_smp;
 }
