@@ -1,4 +1,4 @@
-/* $Id: procmap.c,v 1.14 2003/10/20 20:12:27 hadess Exp $ */
+/* $Id: procmap.c,v 1.18 2004/07/17 12:01:24 bdejean Exp $ */
 
 /* Copyright (C) 1998-99 Martin Baulig
    This file is part of LibGTop 1.0.
@@ -21,10 +21,20 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <glib.h>
+
 #include <glibtop.h>
 #include <glibtop/error.h>
 #include <glibtop/procmap.h>
 
+#include <linux/kdev_t.h>
+
+
+#define PROC_MAPS_FORMAT ((sizeof(void*) == 8) \
+? "%16lx-%16lx %4c %16lx %02hx:%02hx %lu%*[ ]%[^\n]\n" \
+: "%08lx-%08lx %4c %08lx %02hx:%02hx %lu%*[ ]%[^\n]\n")
+
+
 static const unsigned long _glibtop_sysdeps_proc_map =
 (1L << GLIBTOP_PROC_MAP_NUMBER) + (1L << GLIBTOP_PROC_MAP_TOTAL) +
 (1L << GLIBTOP_PROC_MAP_SIZE);
@@ -48,90 +58,80 @@ glibtop_init_proc_map_s (glibtop *server)
 glibtop_map_entry *
 glibtop_get_proc_map_s (glibtop *server, glibtop_proc_map *buf,	pid_t pid)
 {
-	char fn [BUFSIZ];
-	glibtop_map_entry *entry_list = NULL;
-	int rv, n = 0;
+	char procfilename[GLIBTOP_MAP_FILENAME_LEN+1];
+	GArray *entry_list = g_array_new(FALSE, FALSE,
+					 sizeof(glibtop_map_entry));
 	FILE *maps;
 
 	glibtop_init_s (&server, GLIBTOP_SYSDEPS_PROC_MAP, 0);
-	
+
 	memset (buf, 0, sizeof (glibtop_proc_map));
 
-	sprintf (fn, "/proc/%d/maps", pid);
+	snprintf (procfilename, sizeof procfilename, "/proc/%d/maps", pid);
 
-	maps = fopen (fn, "r");
-	if (!maps) return NULL;
+	if((maps = fopen (procfilename, "r")) == NULL) {
+	  return (glibtop_map_entry*) g_array_free(entry_list, TRUE);
+	}
 
-	do {
-		short dev_major, dev_minor;
-		unsigned long start, end, offset, inode, perm;
-		char flags [5], *format;
-		size_t size;
+	while(TRUE)
+	{
+		unsigned long perm = 0;
+		int rv;
 
-		if (sizeof (void*) == 8)
-			format = "%16lx-%16lx %4c\n %16lx %02hx:%02hx %ld";
-		else
-			format = "%08lx-%08lx %4c\n %08lx %02hx:%02hx %ld";
+		unsigned short dev_major, dev_minor;
+		unsigned long start, end, offset, inode;
+		char flags[4];
+		char filename [GLIBTOP_MAP_FILENAME_LEN+1];
 
-		rv = fscanf (maps, format,
+		glibtop_map_entry entry;
+
+
+		/* 8 arguments */
+		rv = fscanf (maps, PROC_MAPS_FORMAT,
 			     &start, &end, flags, &offset,
-			     &dev_major, &dev_minor, &inode);
+			     &dev_major, &dev_minor, &inode, filename);
 
-		flags [4] = 0;
+		if(rv == EOF || rv < 7)
+		  break;
+
+		if(rv == 7) /* no filename */
+		  filename[0] = '\0';
 
 		/* Compute access permissions. */
 
-		perm = 0;
-
 		if (flags [0] == 'r')
 			perm |= GLIBTOP_MAP_PERM_READ;
+
 		if (flags [1] == 'w')
 			perm |= GLIBTOP_MAP_PERM_WRITE;
+
 		if (flags [2] == 'x')
 			perm |= GLIBTOP_MAP_PERM_EXECUTE;
+
 		if (flags [3] == 's')
 			perm |= GLIBTOP_MAP_PERM_SHARED;
-		if (flags [3] == 'p')
+		else if (flags [3] == 'p')
 			perm |= GLIBTOP_MAP_PERM_PRIVATE;
 
-		/* Read filename. */
+		entry.flags = _glibtop_sysdeps_map_entry;
+		entry.start = (guint64) start;
+		entry.end = (guint64) end;
+		entry.offset = (guint64) offset;
+		entry.perm = (guint64) perm;
+		entry.device = (guint64) MKDEV(dev_major, dev_minor);
+		entry.inode = (guint64) inode;
+		g_strlcpy (entry.filename, filename, sizeof entry.filename);
 
-		fn [0] = fgetc (maps);
+		g_array_append_val(entry_list, entry);
+	}
 
-		if (fn [0] != '\n' && fn [0] != EOF) {
-			
-			fscanf (maps, "%*[ ]%[^\n]\n", fn);
-			
-		} else fn [0] = 0;
-
-		size = (n+1) * sizeof (glibtop_map_entry);
-
-		entry_list = g_realloc (entry_list, size);
-
-		memset (&(entry_list [n]), 0, sizeof (glibtop_map_entry));
-
-		entry_list [n].flags = _glibtop_sysdeps_map_entry;
-
-		entry_list [n].start = (guint64) start;
-		entry_list [n].end = (guint64) end;
-		entry_list [n].offset = (guint64) offset;
-		entry_list [n].perm = (guint64) perm;
-		entry_list [n].device = (guint64) (dev_major << 8) +
-			(guint64) dev_minor;
-		entry_list [n].inode = (guint64) inode;
-
-		strncpy (entry_list [n].filename, fn, GLIBTOP_MAP_FILENAME_LEN);
-		entry_list [n].filename [GLIBTOP_MAP_FILENAME_LEN] = 0;
-		
-		n++;
-
-	} while (rv != EOF && rv && fn [0] != EOF);
-	
 	fclose (maps);
 
-	buf->number = n;
-	buf->size = sizeof (glibtop_map_entry);
-	buf->total = n * sizeof (glibtop_map_entry);
+	buf->flags = _glibtop_sysdeps_proc_map;
 
-	return entry_list;
+	buf->number = entry_list->len;
+	buf->size = sizeof (glibtop_map_entry);
+	buf->total = buf->number * buf->size;
+
+	return (glibtop_map_entry*) g_array_free(entry_list, FALSE);
 }
